
local addon_name, addon = ...

local module = addon:NewModule("SpecialItems", "AceEvent-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("PetLeash")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local ipairs = _G.ipairs

--
--
--

local quest_items = {
    -- Orcish Orphan Whistle
    ["item:18597"] = {
        "BAG_UPDATE",
        function() return GetItemCount(18597) > 0 end
    },
    
    -- Human Orphan Whistle
    ["item:18598"] = {
        "BAG_UPDATE",
        function() return GetItemCount(18598) > 0 end
    },

    -- Bloodsail Hat
    ["item:12185"] = {
        "PLAYER_EQUIPMENT_CHANGED EQUIPMENT_SWAP_FINISHED",
        function() return IsEquippedItem(12185) end
    },

    -- Felhound Whistle (TODO TEST ME)
    ["item:30803"] = {
        "BAG_UPDATE",
        function() return GetItemCount(30803) > 0 end
    },

    -- Zeppit's Crystal (Bloody Imp-ossible!)
    ["item:31815"] = {
        "BAG_UPDATE",
        function() return GetItemCount(31815) > 0 end
    },

    -- Blood Elf Orphan Whistle
    ["item:31880"] = {
        "BAG_UPDATE",
        function() return GetItemCount(31880) > 0 end
    },
    
    -- Draenei Orphan Whistle
    ["item:31881"] = {
        "BAG_UPDATE",
        function() return GetItemCount(31881) > 0 end
    },

    -- Nether Ray Cage (TODO CHECK ME)
    ["item:32834"] = {
        "BAG_UPDATE",
        function() return GetItemCount(32834) > 0 end,
    },

    -- Warsong Flare Gun (Alliance Deserter) (TODO TEST ME)
    ["item:34971"] = {
        "BAG_UPDATE",
        function() return GetItemCount(34971) > 0 end
    },

    -- Golem Control Unit (TODO TEST ME)
    ["item:36936"] = {
        "BAG_UPDATE",
        function() return GetItemCount(36936) > 0 end
    },
   
    -- Don Carlos' Famous Hat
    ["item:38506"] = {
        "PLAYER_EQUIPMENT_CHANGED EQUIPMENT_SWAP_FINISHED",
        function() return IsEquippedItem(38506) end
    },

    -- Venomhide Hatchling (20day Raptor Mount quest)
    ["item:46362"] = {
        "BAG_UPDATE ZONE_CHANGED ZONE_CHANGED_NEW_AREA",
        function()
            local mapname = GetZoneText()
            return GetItemCount(46362) > 0 and
                   (mapname == GetMapNameByID(201) or
                    mapname == GetMapNameByID(161) or
                    mapname == GetMapNameByID(261))
        end
    },

    -- Wolvar Orphan Whistle
    ["item:46396"] = {
        "BAG_UPDATE",
        function() return GetItemCount(46396) > 0 end
    },
    
    -- Oracle Orphan Whistle
    ["item:46397"] = {
        "BAG_UPDATE",
        function() return GetItemCount(46397) > 0 end
    },
    
    -- TODO check me
    ["item:46831"] = {
        "BAG_UPDATE",
        function() return GetItemCount(46831) > 0 end,
    },

    -- Winterspring Cub (20day Winterspring Frostsaber quest) 
    ["item:68646"] = {
        "BAG_UPDATE ZONE_CHANGED ZONE_CHANGED_NEW_AREA",
        function()
            local mapname = GetZoneText()
            return GetItemCount(68646) > 0 and mapname == GetMapNameByID(281)
        end
    },

    ["item:71137"] = {
        "BAG_UPDATE",
        function() return GetItemCount(71137) > 0 end
    },

    -- TODO check me
    ["quest:11878"] = {
        "_QUEST",
        function() return module.currentQuests[11878] end
    },

    -- TODO check me
    ["quest:26831"] = {
        "_QUEST",
        function() return module.currentQuests[26831] end
    },
    
    -- TODO check me
    ["quest:25371"] = {
        "_QUEST",
        function() return module.currentQuests[25371] end,
    },
}

local defaults = {
    profile = {
        enable = true,
        items = {
            ["*"] = true
        }
    },
    global = {
        localeCache = {
            ["*"] = {}
        }
    }
}

local options = {
    type = "group",
    name = L["Special Items"],
    order = 11,
    cmdHidden = true,
    args = {
        explain = {
            type = "description",
            name = L["EXPLAIN_SPECIAL_ITEMS"],
            order = 0,
        },
        enable = {
            type = "toggle",
            name = ENABLE,
            order = 1,
            get = function()
                return module.db.profile.enable
            end,
            set = function(info, val)
                module.db.profile.enable = val
                module:UpdateReadyEnabled()
            end,
        },
        itemsGroup = {
            type = "group",
            name = ITEMS,
            inline = true,
            order = 2,
            get = function(info)
                return module.db.profile.items[info[#info]]
            end,
            set = function(info, val)
                module.db.profile.items[info[#info]] = val
                module:UpdateReadyEnabled()
            end,
            args = {},
        },
    }
}


--
--
--

function module:OnInitialize()
    self.event_funcs = {}
    self.event_handler = {}
    LibStub("AceEvent-3.0"):Embed(self.event_handler)  

    self.ready = addon:GetModule("Ready")
    self.db = addon.db:RegisterNamespace("SpecialItems", defaults)

    self:UpdateReadyEnabled()

    self.currentQuests = {}
    self:UpdateQuestList()

    self:RegisterEvent("QUEST_ACCEPTED", "UpdateQuestList")
    self:RegisterEvent("QUEST_FINISHED", "UpdateQuestList")
end

function module:OnEnable()
    self:UpdateWatchers()
end

function module:SetupOptions(setup)
    AceConfig:RegisterOptionsTable(self.name, options)
    self:UpdateConfigTables()
    setup(self.name, L["Special Items"])
end

function module:UpdateConfigTables()
    return self:UpdateWatchers()
end

function module:UpdateReadyEnabled()
    for item, info in pairs(quest_items) do
        self.ready:EnableCheck(item, module.db.profile.enable and module.db.profile.items[item])
    end
end

local function config_localize_string(info)
    return module:LocalizeString(info[#info])
end

function module:UpdateWatchers()
    local args = options.args.itemsGroup.args

    wipe(args)
    for item, info in pairs(quest_items) do
        args[item] = {
            type = "toggle",
            name = config_localize_string,
        }

        local func = info[2]
        if not info.registered then
            info.registered = true
            self:_RegisterEvents(item, func, "PLAYER_ENTERING_WORLD", strsplit(" ", info[1]))
        end
        self:UpdateSpecialItem(item, func("UpdateWatchers"))
    end
end

function module:_RegisterEvents(item, func, ...)
    for ei = 1,select("#", ...) do
        local ev = select(ei, ...)

        if not self.event_handler[ev] then
            self.event_funcs[ev] = {}
            self.event_handler[ev] = function(handler, ...)
                for _,func in ipairs(self.event_funcs[ev]) do
                    func(...)
                end
            end
            if strsub(ev, 1, 1) ~= "_" then
                self.event_handler:RegisterEvent(ev)
            end
        end

        tinsert(self.event_funcs[ev], function(...)
            self:UpdateSpecialItem(item, func(...))
        end)
    end
end

function module:UpdateSpecialItem(item, value)
    self.ready:SetReady(item, not value)
end

function module:UpdateQuestList()
    wipe(self.currentQuests)
    for i = 1, (GetNumQuestLogEntries() or 0) do
        local link = GetQuestLink(i)
        if link ~= nil then
            local _,_,qid = string.find(link, "|Hquest:(%d+):(%d+)|")
            if qid ~= nil then
                self.currentQuests[tonumber(qid)] = true
            end
       end
    end

    if self.event_handler._QUEST then
        self.event_handler:_QUEST()
    end
end

function module:LocalizeString(s)
    local cache = self.db.global.localeCache[GetLocale()]
    if cache[s] then
        return cache[s]
    end

    local name = self:AskClientForName(s)
    if name then
        cache[s] = name
        return name
    end

    return s
end

function module:AskClientForName(item)
    if strsub(item, 1,5) == "item:" then
        return (GetItemInfo(item))
    elseif strsub(item, 1,6) == "spell:" then
        return (GetSpellInfo(strsub(item,7)))
    elseif strsub(item, 1,6) == "quest:" then
        local questid = strsub(item,7)
        for i = 1, (GetNumQuestLogEntries() or 0) do
            local link = GetQuestLink(i)
            if link ~= nil then
                local _,_,qid = string.find(link, "|Hquest:(%d+):(%d+)|")
                if qid == questid then
                    return (GetQuestLogTitle(i))
                end
           end
        end
    end
end

